<div>
    {{ form.media }}

    {{ bs_form_start(form) }}

    {{ form.non_field_errors() }}

    {% for field in form %}
        {{ bs_field(field) }}
    {% endfor %}

    <div class="form-actions">
        <button type="submit" class="btn btn-primary">
            <i class="fa fa-save"></i> {{ button_submit_label or "Save" }}
        </button>
        <button type="reset" class="btn btn-warning">
            <i class="fa fa-eraser"></i> {{ button_reset_label or "Reset" }}
        </button>
    </div>

    {{ bs_form_end(form) }}
</div>